/*
 * impopen.c - loader for library
 *
 * Copyright (C) 2019 Thorsten Otto
 */
#include "config.h"
#include "slb/exif.h"
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <time.h>
#include <errno.h>
#include <mint/cookie.h>
#include <mint/mintbind.h>
#include <math.h>
#include <setjmp.h>
#include "libstrct.h"
#include "slbload.h"
#include "libexif/exifutil.h"
#include "libexif/exifdata.h"
#include "vernum.h"


struct _exif_funcs exif_funcs;


typedef struct
{
	long key;
	long value;
} COOKIE;


static COOKIE *get_cookie_jar(void)
{
	return (COOKIE *) Setexc(360, (void (*)()) -1L);
}


static int get_cookie(long key, long *value)
{
	COOKIE *cookiejar = get_cookie_jar();

	*value = 0;
	if (cookiejar)
	{
		while (cookiejar->key)
		{
			if (cookiejar->key == key)
			{
				*value = cookiejar->value;
				return 1;
			}
			cookiejar++;
		}
	}
	return 0;
}


#if defined(__MSHORT__) || defined(__PUREC__) || defined(__AHCC__)

static void *__CDECL gcc_memset(void *s, exif_int_t val, size_t len)
{
	return memset(s, (int)val, len);
}

static void *__CDECL gcc_memcpy(void *d, const void *s, size_t len)
{
	return memcpy(d, s, len);
}

static void *__CDECL gcc_memchr(const void *s, exif_int_t val, size_t len)
{
	return memchr(s, (int)val, len);
}

static exif_int_t __CDECL gcc_memcmp(const void *s, const void *d, size_t len)
{
	return memcmp(s, d, len);
}

static size_t __CDECL gcc_strlen(const char *s)
{
	return strlen(s);
}

static char *__CDECL gcc_strcpy(char *d, const char *s)
{
	return strcpy(d, s);
}

static char *__CDECL gcc_strncpy(char *d, const char *s, size_t len)
{
	return strncpy(d, s, len);
}

static char *__CDECL gcc_strcat(char *d, const char *s)
{
	return strcat(d, s);
}

static char *__CDECL gcc_strncat(char *d, const char *s, size_t len)
{
	return strncat(d, s, len);
}

static exif_int_t __CDECL gcc_strcmp(const char *d, const char *s)
{
	return strcmp(d, s);
}

static exif_int_t __CDECL gcc_strncmp(const char *d, const char *s, size_t len)
{
	return strncmp(d, s, len);
}

static void *__CDECL gcc_malloc(size_t s)
{
	return malloc(s);
}

static void *__CDECL gcc_calloc(size_t n, size_t s)
{
	return calloc(n, s);
}

static void *__CDECL gcc_realloc(void *p, size_t s)
{
	return realloc(p, s);
}

static void __CDECL gcc_free(void *s)
{
	free(s);
}

#if defined(__PUREC__) || defined(__AHCC__)
#define ESRCH	3	    /* No such process            */
#define EINTR	4		/* Interrupted system call    */
#define ENXIO	6		/* No such device or address  */
#define EFAULT	14		/* Bad address                */
#define ENOTBLK	15		/* Bulk device required       */
#define EBUSY	16		/* Resource is busy           */
#define ENOTTY	25		/* Not a terminal             */
#define ETXTBSY	26		/* Text file is busy          */
#define EFBIG	27		/* File is too large          */
#define EMLINK	31		/* Too many links             */
#define EPIPE	32		/* Broken pipe                */
#define EILSEQ  84      /* Illegal byte sequence      */
#endif

static FILE *__CDECL gcc_fopen(const char *fname, const char *mode)
{
	return fopen(fname, mode);
}

static exif_int_t __CDECL gcc_fclose(FILE *fp)
{
	return fclose(fp);
}

static size_t __CDECL gcc_fread(void *buf, size_t size, size_t count, FILE *fp)
{
	return fread(buf, size, count, fp);
}

static exif_int_t __CDECL gcc_sprintf(char *buf, const char *format, ...)
{
	va_list args;
	int ret;
	va_start(args, fmt);
	ret = vsprintf(buf, format, args);
	va_end(args);
	return ret;
}

static exif_int_t __CDECL gcc_snprintf(char *buf, size_t len,const char *format, ...)
{
	va_list args;
	int ret;
	/* FIXME: no snprintf in pcstdlib.lib */
	(void)len;
	va_start(args, fmt);
	ret = vsprintf(buf, format, args);
	va_end(args);
	return ret;
}

static exif_int_t __CDECL gcc_printf(const char *format, ...)
{
	va_list args;
	int ret;
	va_start(args, fmt);
	ret = vfprintf(stdout, format, args);
	va_end(args);
	return ret;
}

#include "qsort.c"
#include "bsearch.c"

static struct tm *pc_to_minttm(struct tm *tm)
{
	static struct {
		long tm_sec;			/* Seconds.	[0-60] (1 leap second) */
		long tm_min;			/* Minutes.	[0-59] */
		long tm_hour;			/* Hours.	[0-23] */
		long tm_mday;			/* Day.		[1-31] */
		long tm_mon;			/* Month.	[0-11] */
		long tm_year;			/* Year	- 1900.  */
		long tm_wday;			/* Day of week.	[0-6] */
		long tm_yday;			/* Days in year.[0-365]	*/
		long tm_isdst;			/* DST.		[-1/0/1]*/
		long int tm_gmtoff;		/* Seconds east of UTC.  */
		const char *tm_zone;	/* Timezone abbreviation.  */
	} minttm;
	minttm.tm_sec = tm->tm_sec;
	minttm.tm_min = tm->tm_min;
	minttm.tm_hour = tm->tm_hour;
	minttm.tm_mday = tm->tm_mday;
	minttm.tm_mon = tm->tm_mon;
	minttm.tm_year = tm->tm_year;
	minttm.tm_wday = tm->tm_wday;
	minttm.tm_yday = tm->tm_yday;
	minttm.tm_isdst = tm->tm_isdst;
	minttm.tm_gmtoff = timezone;
	return (struct tm *)&minttm;
}

static struct tm *__CDECL gcc_localtime(const time_t *t)
{
	struct tm *tm;
	tm = localtime(t);
	if (tm == NULL)
		return tm;
	return pc_to_minttm(tm);
}

#define S(x) exif_funcs.p_##x = gcc_##x

static time_t __CDECL gcc_time(time_t *tloc)
{
	return time(tloc);
}

#else

#define S(x) exif_funcs.p_##x = x

#endif


long slb_exif_open(const char *slbpath)
{
	long ret;
	long flags;
	long cpu;
	SLB *exif = slb_exif_get();
	
	if (!exif)
		return -EBADF;
	if (exif->handle)
		return 0;

	exif_funcs.struct_size = sizeof(exif_funcs);
	exif_funcs.int_size = sizeof(exif_int_t);
	exif_funcs.exif_vernum = EXIF_VERSION;

	S(memset);
	S(memcpy);
	S(memcmp);
	S(memchr);

	S(strlen);
	S(strcpy);
	S(strncpy);
	S(strcat);
	S(strncat);
	S(strcmp);
	S(strncmp);

	S(malloc);
	S(calloc);
	S(realloc);
	S(free);

	S(fopen);
	S(fclose);
	S(fread);
	S(sprintf);
	S(snprintf);
	S(printf);

	S(qsort);
	S(bsearch);

	S(time);
	S(localtime);
#undef S

	ret = slb_load(EXIF_SHAREDLIB_NAME, slbpath, EXIF_VERSION, &exif->handle, &exif->exec);
	if (ret < 0)
	{
		slb_exif_close();
		return ret;
	}

	/*
	 * check compile flags; that function should be as simple as to just return a constant
	 * and we can hopefully call it even on mismatched configurations
	 */
	flags = exif_slb_control(0, 0);
	get_cookie(C__CPU, &cpu);
	if (cpu >= 20)
	{
		/* should be able to use a 000 library, anyways */
	} else
	{
		if (flags & (1L << 16))
		{
			/* cpu is not 020+, but library was compiled for it */
			slb_exif_close();
			return -EINVAL;
		}
	}
#if defined(__mcoldfire__)
	/* if cpu is cf, but library was not compiled for it... */
	if (!(flags & (1L << 17)))
#else
	/* if cpu is not cf, but library was compiled for it... */
	if (flags & (1L << 17))
#endif
	{
		slb_exif_close();
		return -EINVAL;
	}
	
	ret = exif_slb_control(1, &exif_funcs);
	if (ret < 0)
	{
		slb_exif_close();
		return ret;
	}
	
	return ret;
}
