/* exif-utils.c
 *
 * Copyright (c) 2001 Lutz Mueller <lutz@users.sourceforge.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details. 
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301  USA.
 */

#include <config.h>
#include <stdio.h>
#include <stdarg.h>

#include <libexif/exifutil.h>

#ifdef __PUREC__
struct _ExifEntryPrivate { int dummy; };
struct _ExifContentPrivate { int dummy; };
struct _ExifMnoteData { int dummy; };
struct _ExifDataPrivate { int dummy; };
struct _ExifLog { int dummy; };
struct _ExifMem { int dummy; };
#endif

EXIF_API(void) exif_array_set_byte_order (exif_enum(ExifFormat) f, unsigned char *b, uint32_t n, exif_enum(ExifByteOrder) o_orig, exif_enum(ExifByteOrder) o_new)
{
	uint32_t j;
	uint32_t fs = exif_format_get_size (f);
	ExifShort s;
	ExifSShort ss;
	ExifLong l;
	ExifSLong sl;
	ExifRational r;
	ExifSRational sr;

	if (!b || !n || !fs) return;

	switch ((int)f) {
	case EXIF_FORMAT_SHORT:
		for (j = 0; j < n; j++) {
			s = exif_get_short (b + j * fs, o_orig);
			exif_set_short (b + j * fs, o_new, s);
		}
		break;
	case EXIF_FORMAT_SSHORT:
		for (j = 0; j < n; j++) {
			ss = exif_get_sshort (b + j * fs, o_orig);
			exif_set_sshort (b + j * fs, o_new, ss);
		}
		break;
	case EXIF_FORMAT_LONG:
		for (j = 0; j < n; j++) {
			l = exif_get_long (b + j * fs, o_orig);
			exif_set_long (b + j * fs, o_new, l);
		}
		break;
	case EXIF_FORMAT_RATIONAL:
		for (j = 0; j < n; j++) {
			r = exif_get_rational (b + j * fs, o_orig);
			exif_set_rational (b + j * fs, o_new, &r);
		}
		break;
	case EXIF_FORMAT_SLONG:
		for (j = 0; j < n; j++) {
			sl = exif_get_slong (b + j * fs, o_orig);
			exif_set_slong (b + j * fs, o_new, sl);
		}
		break;
	case EXIF_FORMAT_SRATIONAL:
		for (j = 0; j < n; j++) {
			sr = exif_get_srational (b + j * fs, o_orig);
			exif_set_srational (b + j * fs, o_new, &sr);
		}
		break;
	case EXIF_FORMAT_UNDEFINED:
	case EXIF_FORMAT_BYTE:
	case EXIF_FORMAT_ASCII:
	default:
		/* Nothing here. */
		break;
	}
}

EXIF_API(ExifSShort) exif_get_sshort (const unsigned char *buf, exif_enum(ExifByteOrder) order)
{
	if (!buf) return 0;
        switch ((int)order) {
        case EXIF_BYTE_ORDER_MOTOROLA:
                return ((buf[0] << 8) | buf[1]);
        case EXIF_BYTE_ORDER_INTEL:
                return ((buf[1] << 8) | buf[0]);
        }

	/* Won't be reached */
	return (0);
}

EXIF_API(ExifShort) exif_get_short (const unsigned char *buf, exif_enum(ExifByteOrder) order)
{
	return (exif_get_sshort (buf, order) & 0xffff);
}

EXIF_API(void) exif_set_sshort (unsigned char *b, exif_enum(ExifByteOrder) order, exif_int_t value)
{
	if (!b) return;
	switch ((int)order) {
	case EXIF_BYTE_ORDER_MOTOROLA:
		b[0] = (unsigned char) (value >> 8);
		b[1] = (unsigned char) value;
		break;
	case EXIF_BYTE_ORDER_INTEL:
		b[0] = (unsigned char) value;
		b[1] = (unsigned char) (value >> 8);
		break;
	}
}

EXIF_API(void) exif_set_short (unsigned char *b, exif_enum(ExifByteOrder) order, exif_uint_t value)
{
	exif_set_sshort (b, order, value);
}

EXIF_API(ExifSLong) exif_get_slong (const unsigned char *b, exif_enum(ExifByteOrder) order)
{
	if (!b) return 0;
        switch ((int)order) {
        case EXIF_BYTE_ORDER_MOTOROLA:
                return ((int32_t)(int8_t)b[0] << 24) | ((uint32_t)b[1] << 16) | ((uint32_t)b[2] << 8) | (uint32_t)b[3];
        case EXIF_BYTE_ORDER_INTEL:
                return ((int32_t)(int8_t)b[3] << 24) | ((uint32_t)b[2] << 16) | ((uint32_t)b[1] << 8) | (uint32_t)b[0];
        }

	/* Won't be reached */
	return (0);
}

EXIF_API(void) exif_set_slong (unsigned char *b, exif_enum(ExifByteOrder) order, ExifSLong value)
{
	if (!b) return;
	switch ((int)order) {
	case EXIF_BYTE_ORDER_MOTOROLA:
		b[0] = (unsigned char) (value >> 24);
		b[1] = (unsigned char) (value >> 16);
		b[2] = (unsigned char) (value >> 8);
		b[3] = (unsigned char) value;
		break;
	case EXIF_BYTE_ORDER_INTEL:
		b[3] = (unsigned char) (value >> 24);
		b[2] = (unsigned char) (value >> 16);
		b[1] = (unsigned char) (value >> 8);
		b[0] = (unsigned char) value;
		break;
	}
}

EXIF_API(ExifLong) exif_get_long (const unsigned char *buf, exif_enum(ExifByteOrder) order)
{
        return exif_get_slong (buf, order) & 0xffffffffUL;
}

EXIF_API(void) exif_set_long (unsigned char *b, exif_enum(ExifByteOrder) order, ExifLong value)
{
	exif_set_slong (b, order, value);
}

EXIF_API(ExifSRational) exif_get_srational (const unsigned char *buf, exif_enum(ExifByteOrder) order)
{
	ExifSRational r;

	r.numerator   = buf ? exif_get_slong (buf, order) : 0;
	r.denominator = buf ? exif_get_slong (buf + 4, order) : 0;

	return (r);
}

EXIF_API(ExifRational) exif_get_rational (const unsigned char *buf, exif_enum(ExifByteOrder) order)
{
	ExifRational r;

	r.numerator   = buf ? exif_get_long (buf, order) : 0;
	r.denominator = buf ? exif_get_long (buf + 4, order) : 0;

	return (r);
}

EXIF_API(void) exif_set_rational (unsigned char *buf, exif_enum(ExifByteOrder) order, const ExifRational *value)
{
	if (!buf) return;
	exif_set_long (buf, order, value->numerator);
	exif_set_long (buf + 4, order, value->denominator);
}

EXIF_API(void) exif_set_srational (unsigned char *buf, exif_enum(ExifByteOrder) order, const ExifSRational *value)
{
	if (!buf) return;
	exif_set_slong (buf, order, value->numerator);
	exif_set_slong (buf + 4, order, value->denominator);
}

/*! This function converts rather UCS-2LE than UTF-16 to UTF-8.
 * It should really be replaced by iconv().
 */
EXIF_API(void) exif_convert_utf16_to_utf8 (char *out, const unsigned short *in, int32_t maxlen)
{
	unsigned short c;

	if (maxlen <= 0) {
		return;
	}
	while ((c = *in++) != 0) {
		/* utf16 actually means utf16-le */
#if !defined(__BYTE_ORDER__) || (__BYTE_ORDER__ == __ORDER_BIG_ENDIAN__)
		c = (c << 8) | (c >> 8);
#endif
		if (c < 0x80) {
			if (maxlen > 1) {
				*out++ = (char)c;
				maxlen--;
			} else {
				break;
			}
		} else if (c < 0x800) {
			if (maxlen > 2) {
				*out++ = ((c >> 6) & 0x1F) | 0xC0;
				*out++ = (c & 0x3F) | 0x80;
				maxlen -= 2;
			} else {
				break;
			}
		} else {
			if (maxlen > 3) {
				*out++ = ((c >> 12) & 0x0F) | 0xE0;
				*out++ = ((c >> 6) & 0x3F) | 0x80;
				*out++ = (c & 0x3F) | 0x80;
				maxlen -= 3;
			} else {
				break;
			}
		}
	}
	*out = 0;
}

#if defined(__PUREC__) && !defined(EXIF_SLB)
snprintf(char *buf, size_t len, const char *format, ...)
{
	va_list args;
	int ret;
	/* FIXME: no snprintf in pcstdlib.lib */
	(void)len;
	va_start(args, fmt);
	ret = vsprintf(buf, format, args);
	va_end(args);
	return ret;
}
#endif
